@@ -1,5 +1,7 @@
 from .api_config import (
     LIGHT_API,
+    CANARY_LIGHT_API,
     LUX_API,
+    CANARY_LUX_API,
 )
 from .submit_machine import Toilmore
