@@ -41,11 +41,13 @@ class RejectionNoticeEnum(Enum):
     HTTP_REQUEST_ERROR = "HTTP_REQUEST_ERROR"
     # Timeout
     TIMEOUT = "TIMEOUT"
+    # Bad request
+    BAD_REQUEST = "BAD_REQUEST"
 
 
 class RejectionNotice(NamedTuple):
     rejection_notice: RejectionNoticeEnum
-    inner_error: Optional[RequestIssueException] = None
+    inner_error: Optional[Union[RequestIssueException, str]] = ""
 
 
 class RejectionNoticeToHumanEnum(Enum):
@@ -240,7 +242,7 @@ class Toilmore(object):
         except RequestIssueException as e:
             rn = RejectionNotice(
                 rejection_notice=RejectionNoticeEnum.HTTP_REQUEST_ERROR,
-                inner_error=e
+                inner_error=str(e)
             )
             return OptimizationResponse(
                 status=OptimizationResponseStatus.FAILURE,
@@ -250,8 +252,11 @@ class Toilmore(object):
         if precursor.value not in allowed_precursors:
             rn = RejectionNotice(
                 rejection_notice=RejectionNoticeEnum.HTTP_REQUEST_ERROR,
-                inner_error=RequestIssueException(
-                    f'WrongPrecursor. Allowed values are: {allowed_precursors}'
+                inner_error=str(
+                    RequestIssueException(
+                        f'WrongPrecursor. Allowed values are: '
+                        f'{allowed_precursors}'
+                    )
                 )
             )
             return OptimizationResponse(
@@ -299,9 +304,10 @@ class Toilmore(object):
         if not one_pass_info.image_type:
             rn = RejectionNotice(
                 rejection_notice=RejectionNoticeEnum.HTTP_REQUEST_ERROR,
-                inner_error=RequestIssueException(
-                    'The image input is not allowed. We only accept: {}'.format(
-                        RecognizedImageType.list()
+                inner_error=str(
+                    RequestIssueException(
+                        f'The image input is not allowed. We only accept: '
+                        f'{RecognizedImageType.list()}'
                     )
                 )
             )
@@ -328,7 +334,7 @@ class Toilmore(object):
             )
             rn = RejectionNotice(
                 rejection_notice=RejectionNoticeEnum.UNCLEAR,
-                inner_error=e
+                inner_error=str(e)
             )
             return OptimizationResponse(
                 status=OptimizationResponseStatus.FAILURE,
@@ -379,6 +385,28 @@ class Toilmore(object):
                 status=OptimizationResponseStatus.FAILURE,
                 rejection_notice=rn
             )
+        if res_status == 400:
+            error_explanation = res0.body.get('explanation', "")
+            error_code = res0.body.get('error_code', "")
+            full_error_msg = f'{error_code}: {error_explanation}'
+            self.emit_debug_message(
+                f'Bad request! {full_error_msg}!'
+            )
+            # {
+            #   'status': 'bad request', 'error_code': 'JSON_VALIDATION_ERROR',
+            #   'explanation': "'lossless' is a required property",
+            #   'properties': {
+            #       'validation_error': "'lossless' is a required property"
+            #   }
+            # }
+            rn = RejectionNotice(
+                rejection_notice=RejectionNoticeEnum.BAD_REQUEST,
+                inner_error=full_error_msg
+            )
+            return OptimizationResponse(
+                status=OptimizationResponseStatus.FAILURE,
+                rejection_notice=rn
+            )
 
         self.emit_debug_message('Got an unexpected response from the API.')
         rn = RejectionNotice(
@@ -473,7 +501,7 @@ class Toilmore(object):
             )
             rn = RejectionNotice(
                 rejection_notice=RejectionNoticeEnum.HTTP_REQUEST_ERROR,
-                inner_error=e
+                inner_error=str(e)
             )
             return OptimizationResponse(
                 status=OptimizationResponseStatus.FAILURE,
@@ -593,7 +621,7 @@ class Toilmore(object):
             self.emit_debug_message('ERROR: HTTP error: {}'.format(str(e)))
             rn = RejectionNotice(
                 rejection_notice=RejectionNoticeEnum.HTTP_REQUEST_ERROR,
-                inner_error=e
+                inner_error=str(e)
             )
             return OptimizationResponse(
                 status=OptimizationResponseStatus.FAILURE,
